"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScanService = void 0;
var context_map_server_client_1 = require("../infrastructure/client/context-map-server.client");
var scan_1 = require("../model/scan");
var resource_service_1 = require("./resource.service");
var technology_stack_service_1 = require("./technology-stack.service");
var actor_service_1 = require("./actor.service");
var ts_morph_1 = require("ts-morph");
var subscribed_endpoint_service_1 = require("./subscribed-endpoint.service");
var languages_service_1 = require("./languages.service");
var glossary_service_1 = require("./glossary.service");
var entity_service_1 = require("./entity.service");
var version_control_service_1 = require("./version-control.service");
var decision_record_service_1 = require("./decision-record.service");
var feature_file_service_1 = require("./feature-file.service");
var contextmapPackageJson = require('../../package.json');
var ScanService = /** @class */ (function () {
    function ScanService(parameters) {
        this.parameters = parameters;
        this.tagCache = [];
        this.decisionRecordsCache = [];
        this.featureFilesCache = [];
        this.project = new ts_morph_1.Project();
        this.client = new context_map_server_client_1.ContextMapServerClient(parameters);
        this.resourceService = new resource_service_1.ResourceService();
        this.technologyStackService = new technology_stack_service_1.TechnologyStackService();
        // this.decoratorService = new DecoratorService(this.project)
        this.actorService = new actor_service_1.ActorService(this.project);
        this.glossaryService = new glossary_service_1.GlossaryService(this.project);
        this.entityService = new entity_service_1.EntityService(this.project);
        this.versionControlService = new version_control_service_1.VersionControlService();
        this.decisionRecordService = new decision_record_service_1.DecisionRecordService(this.project);
        this.featureFileService = new feature_file_service_1.FeatureFileService(this.project);
        this.subscribedEndpointService = new subscribed_endpoint_service_1.SubscribedEndpointService(this.project);
        this.languageService = new languages_service_1.LanguagesService(this.project);
    }
    ScanService.prototype.scan = function () {
        var scan = this.initializeScan();
        this.scanConfiguration(scan);
        this.scanTechStack(scan);
        this.scanActors(scan);
        this.scanSubscribedEndpoints(scan);
        this.scanBytesOfCode(scan);
        this.scanGlossaryTerms(scan);
        this.scanEntities(scan);
        this.scanVersionControl(scan);
        this.scanDecisionRecords(scan);
        this.scanFeatureFiles(scan);
        // this.scanDecorators(scan)
        var optResponse = this.sendScan(scan);
        if (optResponse != null) {
            this.sendDecisionRecordDetails(optResponse.componentId, optResponse.parentComponentId, optResponse.decisionRecordsToSend);
            this.sendFeatureFileDetails(optResponse.componentId, optResponse.parentComponentId, optResponse.featureFilesToSend);
            this.sendReleases(optResponse.componentId, optResponse.parentComponentId, optResponse.releasesToSend);
        }
    };
    ScanService.prototype.initializeScan = function () {
        console.log('Scanning sources at: ' + this.parameters.source);
        try {
            if (typeof this.parameters.source === 'string') {
                this.project.addSourceFilesAtPaths(this.parameters.source);
            }
            else {
                var sources = this.parameters.source;
                this.project.addSourceFilesAtPaths(sources);
            }
        }
        catch (error) {
            console.error('Error while initializing project sourcefiles');
        }
        return (0, scan_1.createScan)(contextmapPackageJson.version);
    };
    ScanService.prototype.scanConfiguration = function (scan) {
        var configurationResource = this.resourceService.scanConfiguration();
        scan.execution.scannedProperties = true;
        scan.execution.scannedSystem = true;
        if (this.parameters.componentName != null && this.parameters.componentName.length > 0) {
            scan.scannedComponent.name = this.parameters.componentName;
        }
        else {
            scan.scannedComponent.name = configurationResource.getName();
        }
        if (this.parameters.systemName != null && this.parameters.systemName.length > 0) {
            scan.scannedComponent.systemName = this.parameters.systemName;
        }
        else {
            scan.scannedComponent.systemName = configurationResource.getSystemName();
        }
        scan.scannedComponent.version = configurationResource.getVersion();
        scan.scannedComponent.team = configurationResource.getTeam();
        scan.scannedComponent.domainVisionStatement = configurationResource.getDomainVisionStatement();
        scan.scannedComponent.type = configurationResource.getComponentType();
        scan.scannedComponent.urlBuildPipeline = configurationResource.getUrlBuildPipeline();
        scan.scannedComponent.urlDocumentation = configurationResource.getUrlDocumentation();
        scan.scannedComponent.urlIssueManagement = configurationResource.getUrlIssueManagement();
        scan.scannedComponent.urlVersionControl = configurationResource.getUrlVersionControl();
    };
    // private scanDecorators(scan: Scan): void {
    //   console.info("Scanning entities");
    //   scan.execution.scannedEntities = true;
    //   scan.entities = this.decoratorService.scan();
    //   console.info(" < Found " + scan.entities.length + " entities");
    // }
    ScanService.prototype.scanBytesOfCode = function (scan) {
        console.info("Scanning bytes of code");
        scan.execution.scannedBytesOfCode = true;
        scan.scannedComponent.bytesOfCodes = this.languageService.getBytesOfCodePerLanguage();
        console.info(" < Found " + scan.scannedComponent.bytesOfCodes.length + " languages");
    };
    ScanService.prototype.scanGlossaryTerms = function (scan) {
        console.info("Scanning glossary terms");
        scan.execution.scannedGlossary = true;
        scan.glossaryTerms = this.glossaryService.scan();
        console.info(" < Found " + scan.glossaryTerms.length + " glossary terms");
    };
    ScanService.prototype.scanDecisionRecords = function (scan) {
        console.info("Scanning for architecture decision records");
        scan.execution.scannedDecisionRecords = true;
        this.decisionRecordsCache = this.decisionRecordService.scan();
        scan.decisionRecordOverviews = this.decisionRecordsCache.map(function (o) { return o.decisionRecordOverview; });
        console.info(" < Found " + scan.decisionRecordOverviews.length + " decision records");
    };
    ScanService.prototype.scanFeatureFiles = function (scan) {
        console.info("Scanning for feature files");
        scan.execution.scannedFeatureFiles = true;
        this.featureFilesCache = this.featureFileService.scan();
        scan.featureFileOverviews = this.featureFilesCache.map(function (o) { return o.featureFileOverview; });
        console.info(" < Found " + scan.featureFileOverviews.length + " feature files");
    };
    ScanService.prototype.scanVersionControl = function (scan) {
        console.info("Scanning version control");
        var versionControlScan = this.versionControlService.scan();
        scan.execution.scannedCommitLog = true;
        scan.commitDetails = versionControlScan.commitDetailList;
        scan.scannedComponent.recentCommits = versionControlScan.commitOverviewList;
        console.info(" < Found " + scan.commitDetails.length + " recent commits");
        scan.execution.scannedReleases = true;
        this.tagCache = versionControlScan.tagList;
        scan.releaseOverviews = this.tagCache.map(function (tag) {
            var ro = {
                name: tag.name,
                createdOn: (0, version_control_service_1.formatTagCreatedOn)(tag)
            };
            return ro;
        });
        console.info(" < Found " + scan.releaseOverviews.length + " releases");
    };
    ScanService.prototype.scanEntities = function (scan) {
        console.info("Scanning entities");
        scan.execution.scannedEntities = true;
        scan.entities = this.entityService.scan();
        console.info(" < Found " + scan.entities.length + " entities");
    };
    ScanService.prototype.scanSubscribedEndpoints = function (scan) {
        console.info("Scanning subscribed endpoints");
        scan.execution.scannedSubscribedEndpoints = true;
        scan.subscribedEndpoints = this.subscribedEndpointService.scan();
        console.info(" < Found " + scan.subscribedEndpoints.length + " subscribed endpoints");
    };
    ScanService.prototype.scanActors = function (scan) {
        console.info("Scanning actors");
        scan.execution.scannedActors = true;
        scan.actors = this.actorService.scan();
        console.info(" < Found " + scan.actors.length + " actors");
    };
    ScanService.prototype.scanTechStack = function (scan) {
        console.info("Scanning techstack");
        scan.execution.scannedTechnologies = true;
        scan.scannedComponent.technologies = this.technologyStackService.scan();
        console.info(" < Found " + scan.scannedComponent.technologies.length + " technologies");
    };
    ScanService.prototype.sendScan = function (scan) {
        return this.client.processScan(scan);
    };
    ScanService.prototype.sendFeatureFileDetails = function (componentId, parentComponentId, requestedOverviews) {
        var _this = this;
        if (requestedOverviews == null) {
            console.info("Server requested no feature file details");
            return;
        }
        requestedOverviews.map(function (requestedFF) {
            return _this.featureFilesCache.find(function (ff) { return ff.featureFileOverview.contentHash === requestedFF.contentHash; });
        })
            .filter(function (scannedFF) { return scannedFF != null; })
            .map(function (scannedFF) { return _this.featureFileService.createFeatureFileDetail(componentId, parentComponentId, scannedFF); })
            .forEach(function (ffDetail) {
            console.info("Sending details of feature file " + ffDetail.name + " requested by server");
            _this.client.addFeatureFile(ffDetail);
        });
    };
    ScanService.prototype.sendDecisionRecordDetails = function (componentId, parentComponentId, requestedOverviews) {
        var _this = this;
        if (requestedOverviews == null) {
            console.info("Server requested no decision records details");
            return;
        }
        requestedOverviews.map(function (requestedADR) {
            return _this.decisionRecordsCache.find(function (dr) { return dr.decisionRecordOverview.contentHash === requestedADR.contentHash; });
        })
            .filter(function (scannedADR) { return scannedADR != null; })
            .map(function (scannedADR) { return _this.decisionRecordService.createDecisionRecordDetail(componentId, parentComponentId, scannedADR); })
            .forEach(function (adrDetail) {
            console.info("Sending details of decision record " + adrDetail.name + " requested by server");
            _this.client.addDecisionRecord(adrDetail);
        });
    };
    ScanService.prototype.sendReleases = function (componentId, parentComponentId, requestedReleases) {
        var _this = this;
        var requestedTagNames = new Set();
        requestedReleases
            .map(function (r) { return r.name; })
            .forEach(function (n) { return requestedTagNames.add(n); });
        this.tagCache
            .filter(function (tag) { return requestedTagNames.has(tag.name); })
            .forEach(function (tag) {
            var releaseDetail = {
                componentId: componentId,
                parentComponentId: parentComponentId,
                name: tag.name,
                createdOn: (0, version_control_service_1.formatTagCreatedOn)(tag),
                commitDetails: tag.commits,
            };
            console.log('Sending details of release ' + tag.name + ' requested by server');
            _this.client.addRelease(releaseDetail);
        });
    };
    return ScanService;
}());
exports.ScanService = ScanService;
