"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatTagCreatedOn = exports.VersionControlService = void 0;
var async_call_blocker_1 = require("../infrastructure/async-call-blocker");
var date_functions_1 = require("../infrastructure/date.functions");
var git = require('isomorphic-git');
var fs = require('fs');
var VersionControlService = /** @class */ (function () {
    function VersionControlService() {
    }
    VersionControlService.prototype.scan = function () {
        var _a, _b, _c;
        var scan = {
            commitOverviewList: [],
            commitDetailList: [],
            tagList: [],
        };
        var gitRootDir = this.findGitRootDir();
        (_a = scan.commitDetailList).push.apply(_a, this.getCommits(gitRootDir));
        (_b = scan.commitOverviewList).push.apply(_b, this.convertDetailsToOverviews(scan.commitDetailList));
        (_c = scan.tagList).push.apply(_c, this.getTags(gitRootDir));
        return scan;
    };
    VersionControlService.prototype.convertDetailsToOverviews = function (details) {
        var dayToCounterMap = new Map();
        details.forEach(function (commitDetail) {
            var key = commitDetail.day;
            var commitOverview = dayToCounterMap.get(key);
            if (commitOverview == null) {
                commitOverview = {
                    day: commitDetail.day,
                    amount: 0
                };
                dayToCounterMap.set(key, commitOverview);
            }
            commitOverview.amount = commitOverview.amount + 1;
        });
        var commitOverviews = [];
        dayToCounterMap.forEach(function (value, key) {
            commitOverviews.push(value);
        });
        return commitOverviews;
    };
    VersionControlService.prototype.getCommits = function (dir) {
        var readCommitResults = this.getLogs(dir);
        return this.maptoCommitDetails(readCommitResults);
    };
    VersionControlService.prototype.getTags = function (dir) {
        var _this = this;
        var tagNames = null;
        git.listTags({ fs: fs, dir: dir }).then(function (results) {
            tagNames = results;
        });
        while (!tagNames) {
            (0, async_call_blocker_1.blockExecution)();
        }
        var tags = [];
        tagNames.forEach(function (tagName) {
            var tagObjectId = _this.readTag(tagName, dir);
            var tagCommit = _this.readCommit(tagObjectId, dir);
            var tag = {
                createdOn: (0, date_functions_1.gitTimestampToDate)(tagCommit.commit.committer.timestamp),
                name: tagName,
                objectId: tagCommit.oid,
                commits: [],
            };
            if (tag.name != null && tag.name.indexOf('SNAPSHOT') < 0) {
                tags.push(tag);
            }
        });
        // sort tags from oldest to newest
        tags = tags.sort(function (t1, t2) {
            if (t1.createdOn == null) {
                if (t2.createdOn == null) {
                    return 0;
                }
                else {
                    return 1;
                }
            }
            else if (t2.createdOn == null) {
                return -1;
            }
            return t1.createdOn.getTime() - t2.createdOn.getTime();
        });
        if (tags.length > 0) {
            // get commits for first tag
            var combinedCommitsFromAllPreviousTags = [];
            tags[0].commits = this.getTagCommitsRecentToOld(dir, tags[0].objectId);
            combinedCommitsFromAllPreviousTags.push.apply(combinedCommitsFromAllPreviousTags, tags[0].commits);
            // then get all subsequent tags with changes compared to the previous
            for (var i = 1; i < tags.length; i++) {
                var commitDetails = this.getTagCommitsRecentToOld(dir, tags[i].objectId);
                // TODO fix performance, instead of filter use for loop and break out of it at first match
                // Can not compare with only previous tag, because it could be a tag without any additional commit
                // compared to the one before that. So we check over all combined tags
                // const commitDetailsNotInAnyPreviousTag = commitDetails.filter(cd => {
                //     const match = combinedCommitsFromAllPreviousTags.find(previousTagCD => previousTagCD.shaHash === cd.shaHash)
                //     return match == null
                // })
                tags[i].commits = this.filterCommitsForTag(commitDetails, combinedCommitsFromAllPreviousTags);
                combinedCommitsFromAllPreviousTags.push.apply(combinedCommitsFromAllPreviousTags, tags[i].commits);
            }
        }
        return tags;
    };
    VersionControlService.prototype.filterCommitsForTag = function (commitDetails, combinedCommitsFromAllPreviousTags) {
        var commitDetailsNotInAnyPreviousTag = [];
        var _loop_1 = function (j) {
            var cd = commitDetails[j];
            var match = combinedCommitsFromAllPreviousTags.find(function (previousTagCD) { return previousTagCD.shaHash === cd.shaHash; });
            if (match != null) {
                return { value: commitDetailsNotInAnyPreviousTag };
            }
            else {
                commitDetailsNotInAnyPreviousTag.push(cd);
            }
        };
        for (var j = 0; j < commitDetails.length; j++) {
            var state_1 = _loop_1(j);
            if (typeof state_1 === "object")
                return state_1.value;
        }
        return commitDetailsNotInAnyPreviousTag;
    };
    VersionControlService.prototype.getTagCommitsRecentToOld = function (dir, tagRef) {
        var readCommitResults = null;
        git.log({ fs: fs, dir: dir, ref: tagRef }).then(function (rcr) { return readCommitResults = rcr; });
        while (!readCommitResults) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return this.maptoCommitDetails(readCommitResults);
    };
    VersionControlService.prototype.findGitRootDir = function () {
        var path = null;
        git.findRoot({ fs: fs, filepath: process.cwd() }).then(function (result) { return path = result; });
        while (!path) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return path;
    };
    VersionControlService.prototype.readTag = function (tagName, dir) {
        var tagObjectId = null;
        git.resolveRef({ fs: fs, dir: dir, ref: tagName }).then(function (result) { return tagObjectId = result; });
        while (!tagObjectId) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return tagObjectId;
    };
    VersionControlService.prototype.maptoCommitDetails = function (readCommitResults, limitToRecentCommits) {
        if (limitToRecentCommits === void 0) { limitToRecentCommits = true; }
        var commitDetails = [];
        var maxLengthCommitMessage = 150;
        var maxAgeCommitInDays = 90;
        var maxNrOfCommitsToTrack = 255;
        var dateAfterWhichCommitsAreAllowed = (0, date_functions_1.createDateInPassed)(maxAgeCommitInDays).getTime() / 1000;
        readCommitResults.forEach(function (rc) {
            var addCommit = false;
            var committer = rc.commit.committer;
            if (limitToRecentCommits) {
                if (committer != null
                    && committer.timestamp != null
                    && committer.timestamp > dateAfterWhichCommitsAreAllowed) {
                    addCommit = true;
                }
            }
            else {
                if (committer != null && committer.timestamp != null) {
                    addCommit = true;
                }
            }
            if (addCommit) {
                var commitDetail = {
                    day: (0, date_functions_1.formatTimestamp)(committer.timestamp),
                    commitMessage: rc.commit.message,
                    shaHash: rc.oid,
                };
                if (commitDetail.commitMessage != null && commitDetail.commitMessage.length > maxLengthCommitMessage) {
                    commitDetail.commitMessage = commitDetail.commitMessage.substring(0, maxLengthCommitMessage);
                }
                if (commitDetail.shaHash != null && commitDetail.shaHash.length > 8) {
                    commitDetail.shaHash = commitDetail.shaHash.substring(0, 8);
                }
                commitDetails.push(commitDetail);
            }
        });
        // limit, we don't want to send/store thousands upon thousands of commits
        if (commitDetails.length > maxNrOfCommitsToTrack) {
            commitDetails.splice(maxNrOfCommitsToTrack);
        }
        return commitDetails;
    };
    VersionControlService.prototype.getLogs = function (dir) {
        var results = null;
        git.log({ fs: fs, dir: dir }).then(function (rcrs) { return results = rcrs; });
        while (!results) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return results;
    };
    VersionControlService.prototype.readCommit = function (objectId, dir) {
        var result = null;
        git.readCommit({ fs: fs, dir: dir, oid: objectId }).then(function (rcr) { return result = rcr; });
        while (!result) {
            (0, async_call_blocker_1.blockExecution)();
        }
        return result;
    };
    return VersionControlService;
}());
exports.VersionControlService = VersionControlService;
function formatTagCreatedOn(tag) {
    if (tag.createdOn == null) {
        return null;
    }
    return (0, date_functions_1.formatDateTime)(tag.createdOn);
}
exports.formatTagCreatedOn = formatTagCreatedOn;
